; redimensionne-par-lot.scm
; traitement de toutes les photos d'un dossier

; Dfinition des extensions autorises
(define typeImg
  '("JPG" "jpg" "jpeg")
)

; Fonction de traitement par lot
(define (redimensionne-par-lot nom-repertoire extension x y xx yy q p)

	; Fonction charge de traiter une image
	(define (traiter-image image)
		; recupere le nom de l'image
		(let* (
		       (nom (car (gimp-image-get-name image)))
		       (drawable (car (gimp-image-get-active-layer image))) ; rend l'image active
		       (lari (car (gimp-image-width image)))
		       (haui (car (gimp-image-height image)))
		       (lar x)
		       (hau y)
		       )
		; changement de taille de l image et selection de la zone adaptee
			; XX YY : largeur et hauteur imposee
			(if (and (= xx TRUE) (= yy TRUE))
				(begin
					(if (> (/ lari haui) (/ x y))  ; image originale plus large en ratio
						(begin
						(gimp-image-scale-full image (/ (* lari y) haui) y INTERPOLATION-LANCZOS)
						(gimp-rect-select image (/ (- (/ (* lari y) haui) x) 2) 0 x y REPLACE 0 0)
						)
					)
					(if (<= (/ lari haui) (/ x y))  ; image originale plus haute en ratio
						(begin
						(gimp-image-scale-full image x (/ (* haui x) lari) INTERPOLATION-LANCZOS)
						(gimp-rect-select image 0 (/ (- (/ (* haui x) lari) y) 2) x y REPLACE 0 0)
						)
					)
				)
			)
			; XX : largeur imposee
			(if (and (= xx TRUE) (= yy FALSE))
				(begin
					(if (> (/ lari haui) (/ x y))  ; image originale plus large en ratio
						(begin
						(gimp-image-scale-full image x (/ (* haui x) lari) INTERPOLATION-LANCZOS)
						(gimp-rect-select image 0 0 x (/ (* haui x) lari) REPLACE 0 0)
						)
					)
					(if (<= (/ lari haui) (/ x y))  ; image originale plus haute en ratio - idem XX YY
						(begin
						(gimp-image-scale-full image x (/ (* haui x) lari) INTERPOLATION-LANCZOS)
						(gimp-rect-select image 0 (/ (- (/ (* haui x) lari) y) 2) x y REPLACE 0 0)
						)
					)
				)
			)
			; YY : hauteur imposee
			(if (and (= xx FALSE) (= yy TRUE))
				(begin
					(if (> (/ lari haui) (/ x y))  ; image originale plus large en ratio - idem XX YY
						(begin
						(gimp-image-scale-full image (/ (* lari y) haui) y INTERPOLATION-LANCZOS)
						(gimp-rect-select image (/ (- (/ (* lari y) haui) x) 2) 0 x y REPLACE 0 0)
						)
					)
					(if (<= (/ lari haui) (/ x y))  ; image originale plus haute en ratio
						(begin
						(gimp-image-scale-full image (/ (* lari y) haui) y INTERPOLATION-LANCZOS)
						(gimp-rect-select image 0 0 (/ (* lari y) haui) y REPLACE 0 0)
						)
					)
				)
			)
			; ni XX ni YY : on prend toute l image et on respecte les maxi
			(if (and (= xx FALSE) (= yy FALSE))
				(begin
					(if (> (/ lari haui) (/ x y))  ; image originale plus large en ratio
						(begin
						(gimp-image-scale-full image x (/ (* haui x) lari) INTERPOLATION-LANCZOS)
						(gimp-rect-select image 0 0 x (/ (* haui x) lari) REPLACE 0 0)
						)
					)
					(if (<= (/ lari haui) (/ x y))  ; image originale plus haute en ratio
						(begin
						(gimp-image-scale-full image (/ (* lari y) haui) y INTERPOLATION-LANCZOS)
						(gimp-rect-select image 0 0 (/ (* lari y) haui) y REPLACE 0 0)
						)
					)
				)
			)

		; copie de la selection :
			(gimp-edit-copy-visible image)
		; colle comme nouvelle image : newimage. (le return de cette fonction impose un CAR pour recuperer le nom)
			(let* (
				(newimage (car (gimp-edit-paste-as-new)))
				(drawable (car (gimp-image-get-active-layer newimage))) ; rend l'image active
				)
		; sauvegarde de l'image
				(file-jpeg-save RUN-NONINTERACTIVE ; RUN-NONINTERACTIVE : pas d'affichage de fenetre
				newimage  ; image d'entree
				drawable
				(string-append "gimp/" p nom)
				(string-append "gimp/" p nom)
				q ; JPEG compression level
				0 ; adoucissement
				1 ; optimize 
				0 ; progressive 
				"automatique" ; comment 
				0 ; subsmp (0-4)
				1 ; baseline 
				0 ; restart 
				1 ; dct
				)
			)
		)
	)


	; Fonction charge de traiter un fichier
	(define (traiter-fichier nom-fichier)
		; Ouverture du fichier
		(let* (
			(image  (car (gimp-file-load RUN-NONINTERACTIVE nom-fichier nom-fichier))) ; RUN-NONINTERACTIVE : pas d'affichage de fenetre
			(calque (car (gimp-image-get-active-drawable image)))
			)
			; Traitement de l'image
			(traiter-image image)
			; Fermeture du fichier
			(gimp-image-delete image)
		)
	)

	; boucle de ballayage du dossier
	(let* (
		(extension (list-ref typeImg extension))
		(pattern   (string-append nom-repertoire "/*." extension))
		(fichiers  (cadr (file-glob pattern 0)))
		)
		; Application de la fonction traiter-fichier  chaque fichier
		(map traiter-fichier fichiers)
	)
)

(script-fu-register "redimensionne-par-lot"
  "<Image>/Script-Fu/redimensionne-par-lot"
  "Exemple de traitement par lot"
  "Jacques Malgrange "
  "Jacques Malgrange"
  "2012-08-11"
  ""
  SF-DIRNAME "Repertoire a traiter" ""
  SF-OPTION "Extension" typeImg
  SF-VALUE "Largeur (maxi)" "584"
  SF-VALUE "Hauteur (maxi)" "473"
  SF-TOGGLE "Largeur imposee" FALSE
  SF-TOGGLE "Hauteur imposee" FALSE
  SF-VALUE "Qualite jpg" "0.85"
  SF-STRING "Prefixe images" "gimp-"
)